from _typeshed import StrPath
from lib2to3.pgen2 import grammar
from lib2to3.pgen2.tokenize import _TokenInfo
from typing import IO, Any, Iterable, Iterator, NoReturn, Text

class PgenGrammar(grammar.Grammar): ...

class ParserGenerator:
    filename: StrPath
    stream: IO[Text]
    generator: Iterator[_TokenInfo]
    first: dict[Text, dict[Text, int]]
    def __init__(self, filename: StrPath, stream: IO[Text] | None = ...) -> None: ...
    def make_grammar(self) -> PgenGrammar: ...
    def make_first(self, c: PgenGrammar, name: Text) -> dict[int, int]: ...
    def make_label(self, c: PgenGrammar, label: Text) -> int: ...
    def addfirstsets(self) -> None: ...
    def calcfirst(self, name: Text) -> None: ...
    def parse(self) -> tuple[dict[Text, list[DFAState]], Text]: ...
    def make_dfa(self, start: NFAState, finish: NFAState) -> list[DFAState]: ...
    def dump_nfa(self, name: Text, start: NFAState, finish: NFAState) -> list[DFAState]: ...
    def dump_dfa(self, name: Text, dfa: Iterable[DFAState]) -> None: ...
    def simplify_dfa(self, dfa: list[DFAState]) -> None: ...
    def parse_rhs(self) -> tuple[NFAState, NFAState]: ...
    def parse_alt(self) -> tuple[NFAState, NFAState]: ...
    def parse_item(self) -> tuple[NFAState, NFAState]: ...
    def parse_atom(self) -> tuple[NFAState, NFAState]: ...
    def expect(self, type: int, value: Any | None = ...) -> Text: ...
    def gettoken(self) -> None: ...
    def raise_error(self, msg: str, *args: Any) -> NoReturn: ...

class NFAState:
    arcs: list[tuple[Text | None, NFAState]]
    def __init__(self) -> None: ...
    def addarc(self, next: NFAState, label: Text | None = ...) -> None: ...

class DFAState:
    nfaset: dict[NFAState, Any]
    isfinal: bool
    arcs: dict[Text, DFAState]
    def __init__(self, nfaset: dict[NFAState, Any], final: NFAState) -> None: ...
    def addarc(self, next: DFAState, label: Text) -> None: ...
    def unifystate(self, old: DFAState, new: DFAState) -> None: ...
    def __eq__(self, other: DFAState) -> bool: ...  # type: ignore[override]

def generate_grammar(filename: StrPath = ...) -> PgenGrammar: ...



import sys
from _typeshed import Self
from concurrent.futures._base import Error, Future as _ConcurrentFuture
from typing import Any, Awaitable, Callable, Generator, Iterable, TypeVar
from typing_extensions import Literal, TypeGuard

from .events import AbstractEventLoop

if sys.version_info < (3, 8):
    from concurrent.futures import CancelledError as CancelledError, TimeoutError as TimeoutError

    class InvalidStateError(Error): ...

if sys.version_info >= (3, 7):
    from contextvars import Context

if sys.version_info >= (3, 9):
    from types import GenericAlias

if sys.version_info >= (3, 8):
    __all__ = ("Future", "wrap_future", "isfuture")
elif sys.version_info >= (3, 7):
    __all__ = ("CancelledError", "TimeoutError", "InvalidStateError", "Future", "wrap_future", "isfuture")
else:
    __all__ = ["CancelledError", "TimeoutError", "InvalidStateError", "Future", "wrap_future", "isfuture"]

_T = TypeVar("_T")

# asyncio defines 'isfuture()' in base_futures.py and re-imports it in futures.py
# but it leads to circular import error in pytype tool.
# That's why the import order is reversed.
def isfuture(obj: object) -> TypeGuard[Future[Any]]: ...

if sys.version_info < (3, 7):
    class _TracebackLogger:
        exc: BaseException
        tb: list[str]
        def __init__(self, exc: Any, loop: AbstractEventLoop) -> None: ...
        def activate(self) -> None: ...
        def clear(self) -> None: ...
        def __del__(self) -> None: ...

class Future(Awaitable[_T], Iterable[_T]):
    _state: str
    @property
    def _exception(self) -> BaseException: ...
    _blocking: bool
    @property
    def _log_traceback(self) -> bool: ...
    @_log_traceback.setter
    def _log_traceback(self, val: Literal[False]) -> None: ...
    _asyncio_future_blocking: bool  # is a part of duck-typing contract for `Future`
    def __init__(self, *, loop: AbstractEventLoop | None = ...) -> None: ...
    def __del__(self) -> None: ...
    if sys.version_info >= (3, 7):
        def get_loop(self) -> AbstractEventLoop: ...
        def _callbacks(self: Self) -> list[tuple[Callable[[Self], Any], Context]]: ...
        def add_done_callback(self: Self, __fn: Callable[[Self], Any], *, context: Context | None = ...) -> None: ...
    else:
        @property
        def _callbacks(self: Self) -> list[Callable[[Self], Any]]: ...
        def add_done_callback(self: Self, __fn: Callable[[Self], Any]) -> None: ...
    if sys.version_info >= (3, 9):
        def cancel(self, msg: Any | None = ...) -> bool: ...
    else:
        def cancel(self) -> bool: ...

    def cancelled(self) -> bool: ...
    def done(self) -> bool: ...
    def result(self) -> _T: ...
    def exception(self) -> BaseException | None: ...
    def remove_done_callback(self: Self, __fn: Callable[[Self], Any]) -> int: ...
    def set_result(self, __result: _T) -> None: ...
    def set_exception(self, __exception: type | BaseException) -> None: ...
    def __iter__(self) -> Generator[Any, None, _T]: ...
    def __await__(self) -> Generator[Any, None, _T]: ...
    @property
    def _loop(self) -> AbstractEventLoop: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

def wrap_future(future: _ConcurrentFuture[_T] | Future[_T], *, loop: AbstractEventLoop | None = ...) -> Future[_T]: ...



"""Tests for pytype_source_utils.py."""

import os

from pytype import pytype_source_utils
from pytype.platform_utils import path_utils

import unittest


class PytypeSourceUtilsTest(unittest.TestCase):
  """Test pytype source utilities."""

  def setUp(self):
    super().setUp()
    self.root = path_utils.dirname(__file__)

  def test_pytype_source_dir(self):
    self.assertEqual(self.root, pytype_source_utils.pytype_source_dir())

  def test_get_full_path(self):
    self.assertEqual(
        path_utils.join(self.root, f"foo{os.path.sep}bar"),
        pytype_source_utils.get_full_path(f"foo{os.path.sep}bar"))
    self.assertEqual(
        f"{os.path.sep}foo{os.path.sep}bar",
        pytype_source_utils.get_full_path(f"{os.path.sep}foo{os.path.sep}bar"))

  def test_list_pytype_files(self):
    l = list(pytype_source_utils.list_pytype_files(f"stubs{os.path.sep}stdlib"))
    self.assertIn("_ctypes.pytd", l)
    self.assertIn(f"collections{os.path.sep}__init__.pytd", l)


if __name__ == "__main__":
  unittest.main()



from cryptography.hazmat.backends.interfaces import HashBackend, HMACBackend
from cryptography.hazmat.primitives.hashes import HashAlgorithm
from cryptography.hazmat.primitives.kdf import KeyDerivationFunction

class ConcatKDFHash(KeyDerivationFunction):
    def __init__(self, algorithm: HashAlgorithm, length: int, otherinfo: bytes | None, backend: HashBackend | None = ...): ...
    def derive(self, key_material: bytes) -> bytes: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class ConcatKDFHMAC(KeyDerivationFunction):
    def __init__(
        self,
        algorithm: HashAlgorithm,
        length: int,
        salt: bytes | None,
        otherinfo: bytes | None,
        backend: HMACBackend | None = ...,
    ): ...
    def derive(self, key_material: bytes) -> bytes: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...



from logging import Logger
from typing import Any, Callable, Mapping, Match, Pattern, Sequence

from .rfc7230 import OBS_TEXT as OBS_TEXT, VCHAR as VCHAR

logger: Logger
queue_logger: Logger

def find_double_newline(s: bytes) -> int: ...
def concat(*args: Any) -> str: ...
def join(seq: Any, field: str = ...) -> str: ...
def group(s: Any) -> str: ...

short_days: Sequence[str]
long_days: Sequence[str]
short_day_reg: str
long_day_reg: str
daymap: Mapping[str, int]
hms_reg: str
months: Sequence[str]
monmap: Mapping[str, int]
months_reg: str
rfc822_date: str
rfc822_reg: Pattern[Any]

def unpack_rfc822(m: Match[Any]) -> tuple[int, int, int, int, int, int, int, int, int]: ...

rfc850_date: str
rfc850_reg: Pattern[Any]

def unpack_rfc850(m: Match[Any]) -> tuple[int, int, int, int, int, int, int, int, int]: ...

weekdayname: Sequence[str]
monthname: Sequence[str]

def build_http_date(when: int) -> str: ...
def parse_http_date(d: str) -> int: ...

vchar_re: str
obs_text_re: str
qdtext_re: str
quoted_pair_re: str
quoted_string_re: str
quoted_string: Pattern[Any]
quoted_pair: Pattern[Any]

def undquote(value: str) -> str: ...
def cleanup_unix_socket(path: str) -> None: ...

class Error:
    code: int = ...
    reason: str = ...
    body: str = ...
    def __init__(self, body: str) -> None: ...
    def to_response(self) -> tuple[str, Sequence[tuple[str, str]], str]: ...
    def wsgi_response(self, environ: Any, start_response: Callable[[str, Sequence[tuple[str, str]]], None]) -> str: ...

class BadRequest(Error):
    code: int = ...
    reason: str = ...

class RequestHeaderFieldsTooLarge(BadRequest):
    code: int = ...
    reason: str = ...

class RequestEntityTooLarge(BadRequest):
    code: int = ...
    reason: str = ...

class InternalServerError(Error):
    code: int = ...
    reason: str = ...

class ServerNotImplemented(Error):
    code: int = ...
    reason: str = ...



"""Platform-independent functions for creating temp files & directories."""

import os
import sys
import tempfile

from pytype.platform_utils import path_utils

# A process on windows cannot open a temp file twice without delete=False
if sys.platform == 'win32':

  class NamedTemporaryFile:
    """A cross-platform replacement for tempfile.NamedTemporaryFile."""

    # pylint: disable=redefined-builtin
    def __init__(self,
                 mode='w+b',
                 buffering=-1,
                 encoding=None,
                 newline=None,
                 suffix=None,
                 prefix=None,
                 dir=None,
                 delete=True):
      # pylint: disable=R1732
      self._tempfile = tempfile.NamedTemporaryFile(
          mode=mode,
          buffering=buffering,
          encoding=encoding,
          newline=newline,
          suffix=suffix,
          prefix=prefix,
          dir=dir,
          delete=False)
      self._delete = delete

    def __enter__(self):
      return self._tempfile.__enter__()

    def __exit__(self, *args, **kwargs):
      self._tempfile.__exit__(*args, **kwargs)
      if self._delete:
        os.remove(self._tempfile.name)

    def write(self, s):
      return self._tempfile.write(s)

    def read(self, n):
      return self._tempfile.read(n)

    def seek(self, *args, **kwargs):
      return self._tempfile.seek(*args, **kwargs)

    def close(self):
      return self._tempfile.close()

    @property
    def name(self):
      return self._tempfile.name

  def mkdtemp(*args, **kwargs):
    return path_utils.standardize_return_path(tempfile.mkdtemp(*args, **kwargs))
else:
  NamedTemporaryFile = tempfile.NamedTemporaryFile
  mkdtemp = tempfile.mkdtemp



from _typeshed import Self
from datetime import date, datetime, timedelta
from typing import SupportsFloat, TypeVar, overload

from ._common import weekday

# We need the extra "Self" TypeVar to avoid overlapping __add__/__radd__ complaints from mypy
_SelfT = TypeVar("_SelfT", bound=relativedelta)
_DateT = TypeVar("_DateT", date, datetime)
# Work around attribute and type having the same name.
_weekday = weekday

MO: weekday
TU: weekday
WE: weekday
TH: weekday
FR: weekday
SA: weekday
SU: weekday

class relativedelta(object):
    years: int
    months: int
    days: int
    leapdays: int
    hours: int
    minutes: int
    seconds: int
    microseconds: int
    year: int | None
    month: int | None
    weekday: _weekday | None
    day: int | None
    hour: int | None
    minute: int | None
    second: int | None
    microsecond: int | None
    def __init__(
        self,
        dt1: date | None = ...,
        dt2: date | None = ...,
        years: int | None = ...,
        months: int | None = ...,
        days: int | None = ...,
        leapdays: int | None = ...,
        weeks: int | None = ...,
        hours: int | None = ...,
        minutes: int | None = ...,
        seconds: int | None = ...,
        microseconds: int | None = ...,
        year: int | None = ...,
        month: int | None = ...,
        day: int | None = ...,
        weekday: int | _weekday | None = ...,
        yearday: int | None = ...,
        nlyearday: int | None = ...,
        hour: int | None = ...,
        minute: int | None = ...,
        second: int | None = ...,
        microsecond: int | None = ...,
    ) -> None: ...
    @property
    def weeks(self) -> int: ...
    @weeks.setter
    def weeks(self, value: int) -> None: ...
    def normalized(self: Self) -> Self: ...
    # TODO: use Union when mypy will handle it properly in overloaded operator
    # methods (#2129, #1442, #1264 in mypy)
    @overload
    def __add__(self: _SelfT, other: relativedelta) -> _SelfT: ...  # noqa: Y019
    @overload
    def __add__(self: _SelfT, other: timedelta) -> _SelfT: ...  # noqa: Y019
    @overload
    def __add__(self, other: _DateT) -> _DateT: ...
    @overload
    def __radd__(self: _SelfT, other: relativedelta) -> _SelfT: ...  # noqa: Y019
    @overload
    def __radd__(self: _SelfT, other: timedelta) -> _SelfT: ...  # noqa: Y019
    @overload
    def __radd__(self, other: _DateT) -> _DateT: ...
    @overload
    def __rsub__(self: Self, other: relativedelta) -> Self: ...
    @overload
    def __rsub__(self: Self, other: timedelta) -> Self: ...
    @overload
    def __rsub__(self, other: _DateT) -> _DateT: ...
    def __sub__(self: Self, other: relativedelta) -> Self: ...
    def __neg__(self: Self) -> Self: ...
    def __bool__(self) -> bool: ...
    def __nonzero__(self) -> bool: ...
    def __mul__(self: Self, other: SupportsFloat) -> Self: ...
    def __rmul__(self: Self, other: SupportsFloat) -> Self: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __div__(self: Self, other: SupportsFloat) -> Self: ...
    def __truediv__(self: Self, other: SupportsFloat) -> Self: ...
    def __abs__(self: Self) -> Self: ...
    def __hash__(self) -> int: ...



"""Wraps some path-related functions in `os`, `os.path` and `glob`.

Standardize return path in Windows. Do nothing in Linux.
"""

import functools
import glob as glob_module
import os
import sys

if sys.platform == 'win32':
  import ctypes  # pylint: disable=g-import-not-at-top

  def _short_path_to_long_path(path: str):
    """Convert to long path names in win32."""
    buffer = ctypes.create_unicode_buffer(0)
    required_size = ctypes.windll.kernel32.GetLongPathNameW(path, buffer, 0)

    # GetLongPathNameW returns empty if a path does not exist
    if not required_size:
      return path

    buffer = ctypes.create_unicode_buffer(required_size)
    ctypes.windll.kernel32.GetLongPathNameW(path, buffer, required_size)

    return buffer.value
else:

  def _short_path_to_long_path(path: str):
    return path


def _replace_driver_code(path: str):
  drive, other = os.path.splitdrive(path)
  drive = drive.capitalize()
  return os.path.join(drive, other)


if sys.platform == 'win32':

  def standardize_return_path(path):
    path = _replace_driver_code(path)
    path = _short_path_to_long_path(path)
    return path
else:

  def standardize_return_path(path):
    return path


def _standardize_return_path_wrapper(func):
  """Standardize return path in win32."""
  if sys.platform == 'win32':

    @functools.wraps(func)
    def wrapped(*args, **kwargs):
      path = func(*args, **kwargs)
      return standardize_return_path(path)

    return wrapped
  else:
    return func


splitext = os.path.splitext
glob = glob_module.glob

abspath = _standardize_return_path_wrapper(os.path.abspath)
relpath = _standardize_return_path_wrapper(os.path.relpath)
dirname = _standardize_return_path_wrapper(os.path.dirname)
expanduser = _standardize_return_path_wrapper(os.path.expanduser)
normpath = _standardize_return_path_wrapper(os.path.normpath)
realpath = _standardize_return_path_wrapper(os.path.realpath)
join = _standardize_return_path_wrapper(os.path.join)
getcwd = _standardize_return_path_wrapper(os.getcwd)

exists = os.path.exists
isdir = os.path.isdir
isabs = os.path.isabs
basename = os.path.basename
split = os.path.split
isfile = os.path.isfile

sep = os.path.sep



"""Tests for file_utils.py."""

from pytype import file_utils
from pytype.platform_utils import path_utils

import unittest


class FileUtilsTest(unittest.TestCase):
  """Test file and path utilities."""

  def test_replace_extension(self):
    self.assertEqual("foo.bar", file_utils.replace_extension("foo.txt", "bar"))
    self.assertEqual("foo.bar", file_utils.replace_extension("foo.txt", ".bar"))
    self.assertEqual("a.b.c.bar",
                     file_utils.replace_extension("a.b.c.txt", ".bar"))
    self.assertEqual(
        file_utils.replace_separator("a.b/c.bar"),
        file_utils.replace_extension(
            file_utils.replace_separator("a.b/c.d"), ".bar"))
    self.assertEqual("xyz.bar", file_utils.replace_extension("xyz", "bar"))

  def test_tempdir(self):
    with file_utils.Tempdir() as d:
      filename1 = d.create_file("foo.txt")
      filename2 = d.create_file("bar.txt", "\tdata2")
      filename3 = d.create_file("baz.txt", "data3")
      filename4 = d.create_file(
          file_utils.replace_separator("d1/d2/qqsv.txt"),
          "  data4.1\n  data4.2")
      filename5 = d.create_directory("directory")
      self.assertEqual(filename1, d["foo.txt"])
      self.assertEqual(filename2, d["bar.txt"])
      self.assertEqual(filename3, d["baz.txt"])
      self.assertEqual(filename4,
                       d[file_utils.replace_separator("d1/d2/qqsv.txt")])
      self.assertTrue(path_utils.isdir(d.path))
      self.assertTrue(path_utils.isfile(filename1))
      self.assertTrue(path_utils.isfile(filename2))
      self.assertTrue(path_utils.isfile(filename3))
      self.assertTrue(path_utils.isfile(filename4))
      self.assertTrue(path_utils.isdir(path_utils.join(d.path, "d1")))
      self.assertTrue(path_utils.isdir(path_utils.join(d.path, "d1", "d2")))
      self.assertTrue(path_utils.isdir(filename5))
      self.assertEqual(filename4, path_utils.join(d.path, "d1", "d2",
                                                  "qqsv.txt"))
      for filename, contents in [(filename1, ""),
                                 (filename2, "data2"),  # dedented
                                 (filename3, "data3"),
                                 (filename4, "data4.1\ndata4.2"),  # dedented
                                ]:
        with open(filename) as fi:
          self.assertEqual(fi.read(), contents)
    self.assertFalse(path_utils.isdir(d.path))
    self.assertFalse(path_utils.isfile(filename1))
    self.assertFalse(path_utils.isfile(filename2))
    self.assertFalse(path_utils.isfile(filename3))
    self.assertFalse(path_utils.isdir(path_utils.join(d.path, "d1")))
    self.assertFalse(path_utils.isdir(path_utils.join(d.path, "d1", "d2")))
    self.assertFalse(path_utils.isdir(filename5))

  def test_cd(self):
    with file_utils.Tempdir() as d:
      d.create_directory("foo")
      d1 = path_utils.getcwd()
      with file_utils.cd(d.path):
        self.assertTrue(path_utils.isdir("foo"))
      d2 = path_utils.getcwd()
      self.assertEqual(d1, d2)

  def test_cd_noop(self):
    d = path_utils.getcwd()
    with file_utils.cd(None):
      self.assertEqual(path_utils.getcwd(), d)
    with file_utils.cd(""):
      self.assertEqual(path_utils.getcwd(), d)


class TestPathExpansion(unittest.TestCase):
  """Tests for file_utils.expand_path(s?)."""

  def test_expand_one_path(self):
    full_path = path_utils.join(path_utils.getcwd(), "foo.py")
    self.assertEqual(file_utils.expand_path("foo.py"), full_path)

  def test_expand_two_paths(self):
    full_path1 = path_utils.join(path_utils.getcwd(), "foo.py")
    full_path2 = path_utils.join(path_utils.getcwd(), "bar.py")
    self.assertEqual(file_utils.expand_paths(["foo.py", "bar.py"]),
                     [full_path1, full_path2])

  def test_expand_with_cwd(self):
    with file_utils.Tempdir() as d:
      f = d.create_file("foo.py")
      self.assertEqual(file_utils.expand_path("foo.py", d.path), f)


class TestExpandSourceFiles(unittest.TestCase):
  """Tests for file_utils.expand_source_files."""

  FILES = [
      "a.py",
      file_utils.replace_separator("foo/b.py"),
      file_utils.replace_separator("foo/c.txt"),
      file_utils.replace_separator("foo/bar/d.py"),
      file_utils.replace_separator("foo/bar/baz/e.py")
  ]

  def _test_expand(self, string):
    with file_utils.Tempdir() as d:
      fs = [d.create_file(f) for f in self.FILES]
      pyfiles = [f for f in fs if f.endswith(".py")]
      self.assertCountEqual(
          pyfiles, file_utils.expand_source_files(string, d.path))

  def test_expand_source_files(self):
    self._test_expand(file_utils.replace_separator("a.py foo/c.txt foo"))

  def test_duplicates(self):
    self._test_expand(file_utils.replace_separator("a.py foo/b.py foo foo/bar"))

  def test_cwd(self):
    with file_utils.Tempdir() as d:
      fs = [d.create_file(f) for f in self.FILES]
      pyfiles = [f for f in fs if f.endswith(".py")]
      # cd to d.path and run with just "." as an argument
      with file_utils.cd(d.path):
        self.assertCountEqual(
            pyfiles, file_utils.expand_source_files("."))

  def test_empty(self):
    self.assertEqual(file_utils.expand_source_files(""), set())

  def test_magic(self):
    filenames = ["a.py", file_utils.replace_separator("b/c.py")]
    with file_utils.Tempdir() as d:
      for f in filenames:
        d.create_file(f)
      with file_utils.cd(d.path):
        self.assertEqual(
            file_utils.expand_source_files(
                file_utils.replace_separator("**/*.py")),
            {path_utils.realpath(f) for f in filenames})

  def test_magic_with_cwd(self):
    filenames = ["a.py", file_utils.replace_separator("b/c.py")]
    with file_utils.Tempdir() as d:
      for f in filenames:
        d.create_file(f)
      self.assertEqual(
          file_utils.expand_source_files(
              file_utils.replace_separator("**/*.py"), cwd=d.path),
          {path_utils.join(d.path, f) for f in filenames})

  def test_multiple_magic(self):
    filenames = ["a.py", file_utils.replace_separator("b/c.py")]
    with file_utils.Tempdir() as d:
      for f in filenames:
        d.create_file(f)
      self.assertEqual(
          file_utils.expand_source_files(
              file_utils.replace_separator("*.py b/*.py"), cwd=d.path),
          {path_utils.join(d.path, f) for f in filenames})


class TestExpandHiddenFiles(unittest.TestCase):

  def test_ignore_file(self):
    with file_utils.Tempdir() as d:
      d.create_file(".ignore.py")
      self.assertEqual(file_utils.expand_source_files(".", cwd=d.path), set())

  def test_find_file(self):
    with file_utils.Tempdir() as d:
      d.create_file(".find.py")
      self.assertEqual(
          file_utils.expand_source_files(".*", cwd=d.path),
          {path_utils.join(d.path, ".find.py")})

  def test_ignore_dir(self):
    with file_utils.Tempdir() as d:
      d.create_file(file_utils.replace_separator("d1/.d2/ignore.py"))
      self.assertEqual(
          file_utils.expand_source_files(
              file_utils.replace_separator("d1/**/*"), cwd=d.path), set())

  def test_find_dir(self):
    with file_utils.Tempdir() as d:
      d.create_file(file_utils.replace_separator(".d/find.py"))
      self.assertEqual(
          file_utils.expand_source_files(
              file_utils.replace_separator(".d/**/*"), cwd=d.path),
          {path_utils.join(d.path, ".d", "find.py")})


class TestExpandPythonpath(unittest.TestCase):

  def test_expand(self):
    self.assertEqual(
        file_utils.expand_pythonpath(file_utils.replace_separator("a/b:c/d")), [
            path_utils.join(path_utils.getcwd(), "a", "b"),
            path_utils.join(path_utils.getcwd(), "c", "d")
        ])

  def test_expand_empty(self):
    self.assertEqual(file_utils.expand_pythonpath(""), [])

  def test_expand_current_directory(self):
    self.assertEqual(
        file_utils.expand_pythonpath(file_utils.replace_separator(":a")),
        [path_utils.getcwd(),
         path_utils.join(path_utils.getcwd(), "a")])

  def test_expand_with_cwd(self):
    with file_utils.Tempdir() as d:
      self.assertEqual(
          file_utils.expand_pythonpath(
              file_utils.replace_separator("a/b:c/d"), cwd=d.path), [
                  path_utils.join(d.path, "a", "b"),
                  path_utils.join(d.path, "c", "d")
              ])

  def test_strip_whitespace(self):
    self.assertEqual(
        file_utils.expand_pythonpath(
            file_utils.replace_separator("""
      a/b:
      c/d
    """)), [
        path_utils.join(path_utils.getcwd(), "a", "b"),
        path_utils.join(path_utils.getcwd(), "c", "d")
    ])


class TestExpandGlobpaths(unittest.TestCase):

  def test_expand_empty(self):
    self.assertEqual(file_utils.expand_globpaths([]), [])

  def test_expand(self):
    filenames = ["a.py", file_utils.replace_separator("b/c.py")]
    with file_utils.Tempdir() as d:
      for f in filenames:
        d.create_file(f)
      with file_utils.cd(d.path):
        self.assertEqual(
            file_utils.expand_globpaths(
                [file_utils.replace_separator("**/*.py")]),
            [path_utils.realpath(f) for f in filenames])

  def test_expand_with_cwd(self):
    filenames = ["a.py", file_utils.replace_separator("b/c.py")]
    with file_utils.Tempdir() as d:
      for f in filenames:
        d.create_file(f)
      self.assertEqual(
          file_utils.expand_globpaths([file_utils.replace_separator("**/*.py")],
                                      cwd=d.path),
          [path_utils.join(d.path, f) for f in filenames])


if __name__ == "__main__":
  unittest.main()



import sys
from typing import Any, Callable, ClassVar, Iterable, Iterator, Mapping, Protocol, Union
from typing_extensions import final

if sys.version_info >= (3, 8):
    __all__ = [
        "PickleError",
        "PicklingError",
        "UnpicklingError",
        "Pickler",
        "Unpickler",
        "dump",
        "dumps",
        "load",
        "loads",
        "PickleBuffer",
        "ADDITEMS",
        "APPEND",
        "APPENDS",
        "BINBYTES",
        "BINBYTES8",
        "BINFLOAT",
        "BINGET",
        "BININT",
        "BININT1",
        "BININT2",
        "BINPERSID",
        "BINPUT",
        "BINSTRING",
        "BINUNICODE",
        "BINUNICODE8",
        "BUILD",
        "BYTEARRAY8",
        "DEFAULT_PROTOCOL",
        "DICT",
        "DUP",
        "EMPTY_DICT",
        "EMPTY_LIST",
        "EMPTY_SET",
        "EMPTY_TUPLE",
        "EXT1",
        "EXT2",
        "EXT4",
        "FALSE",
        "FLOAT",
        "FRAME",
        "FROZENSET",
        "GET",
        "GLOBAL",
        "HIGHEST_PROTOCOL",
        "INST",
        "INT",
        "LIST",
        "LONG",
        "LONG1",
        "LONG4",
        "LONG_BINGET",
        "LONG_BINPUT",
        "MARK",
        "MEMOIZE",
        "NEWFALSE",
        "NEWOBJ",
        "NEWOBJ_EX",
        "NEWTRUE",
        "NEXT_BUFFER",
        "NONE",
        "OBJ",
        "PERSID",
        "POP",
        "POP_MARK",
        "PROTO",
        "PUT",
        "READONLY_BUFFER",
        "REDUCE",
        "SETITEM",
        "SETITEMS",
        "SHORT_BINBYTES",
        "SHORT_BINSTRING",
        "SHORT_BINUNICODE",
        "STACK_GLOBAL",
        "STOP",
        "STRING",
        "TRUE",
        "TUPLE",
        "TUPLE1",
        "TUPLE2",
        "TUPLE3",
        "UNICODE",
    ]
else:
    __all__ = [
        "PickleError",
        "PicklingError",
        "UnpicklingError",
        "Pickler",
        "Unpickler",
        "dump",
        "dumps",
        "load",
        "loads",
        "ADDITEMS",
        "APPEND",
        "APPENDS",
        "BINBYTES",
        "BINBYTES8",
        "BINFLOAT",
        "BINGET",
        "BININT",
        "BININT1",
        "BININT2",
        "BINPERSID",
        "BINPUT",
        "BINSTRING",
        "BINUNICODE",
        "BINUNICODE8",
        "BUILD",
        "DEFAULT_PROTOCOL",
        "DICT",
        "DUP",
        "EMPTY_DICT",
        "EMPTY_LIST",
        "EMPTY_SET",
        "EMPTY_TUPLE",
        "EXT1",
        "EXT2",
        "EXT4",
        "FALSE",
        "FLOAT",
        "FRAME",
        "FROZENSET",
        "GET",
        "GLOBAL",
        "HIGHEST_PROTOCOL",
        "INST",
        "INT",
        "LIST",
        "LONG",
        "LONG1",
        "LONG4",
        "LONG_BINGET",
        "LONG_BINPUT",
        "MARK",
        "MEMOIZE",
        "NEWFALSE",
        "NEWOBJ",
        "NEWOBJ_EX",
        "NEWTRUE",
        "NONE",
        "OBJ",
        "PERSID",
        "POP",
        "POP_MARK",
        "PROTO",
        "PUT",
        "REDUCE",
        "SETITEM",
        "SETITEMS",
        "SHORT_BINBYTES",
        "SHORT_BINSTRING",
        "SHORT_BINUNICODE",
        "STACK_GLOBAL",
        "STOP",
        "STRING",
        "TRUE",
        "TUPLE",
        "TUPLE1",
        "TUPLE2",
        "TUPLE3",
        "UNICODE",
    ]

HIGHEST_PROTOCOL: int
DEFAULT_PROTOCOL: int

bytes_types: tuple[type[Any], ...]  # undocumented

class _ReadableFileobj(Protocol):
    def read(self, __n: int) -> bytes: ...
    def readline(self) -> bytes: ...

class _WritableFileobj(Protocol):
    def write(self, __b: bytes) -> Any: ...

if sys.version_info >= (3, 8):
    # TODO: holistic design for buffer interface (typing.Buffer?)
    @final
    class PickleBuffer:
        # buffer must be a buffer-providing object
        def __init__(self, buffer: Any) -> None: ...
        def raw(self) -> memoryview: ...
        def release(self) -> None: ...
    _BufferCallback = Callable[[PickleBuffer], Any] | None
    def dump(
        obj: Any,
        file: _WritableFileobj,
        protocol: int | None = ...,
        *,
        fix_imports: bool = ...,
        buffer_callback: _BufferCallback = ...,
    ) -> None: ...
    def dumps(
        obj: Any, protocol: int | None = ..., *, fix_imports: bool = ..., buffer_callback: _BufferCallback = ...
    ) -> bytes: ...
    def load(
        file: _ReadableFileobj,
        *,
        fix_imports: bool = ...,
        encoding: str = ...,
        errors: str = ...,
        buffers: Iterable[Any] | None = ...,
    ) -> Any: ...
    def loads(
        __data: bytes, *, fix_imports: bool = ..., encoding: str = ..., errors: str = ..., buffers: Iterable[Any] | None = ...
    ) -> Any: ...

else:
    def dump(obj: Any, file: _WritableFileobj, protocol: int | None = ..., *, fix_imports: bool = ...) -> None: ...
    def dumps(obj: Any, protocol: int | None = ..., *, fix_imports: bool = ...) -> bytes: ...
    def load(file: _ReadableFileobj, *, fix_imports: bool = ..., encoding: str = ..., errors: str = ...) -> Any: ...
    def loads(data: bytes, *, fix_imports: bool = ..., encoding: str = ..., errors: str = ...) -> Any: ...

class PickleError(Exception): ...
class PicklingError(PickleError): ...
class UnpicklingError(PickleError): ...

_reducedtype = Union[
    str,
    tuple[Callable[..., Any], tuple[Any, ...]],
    tuple[Callable[..., Any], tuple[Any, ...], Any],
    tuple[Callable[..., Any], tuple[Any, ...], Any, Iterator[Any] | None],
    tuple[Callable[..., Any], tuple[Any, ...], Any, Iterator[Any] | None, Iterator[Any] | None],
]

class Pickler:
    fast: bool
    dispatch_table: Mapping[type, Callable[[Any], _reducedtype]]
    bin: bool  # undocumented
    dispatch: ClassVar[dict[type, Callable[[Unpickler, Any], None]]]  # undocumented, _Pickler only

    if sys.version_info >= (3, 8):
        def __init__(
            self,
            file: _WritableFileobj,
            protocol: int | None = ...,
            *,
            fix_imports: bool = ...,
            buffer_callback: _BufferCallback = ...,
        ) -> None: ...
        def reducer_override(self, obj: Any) -> Any: ...
    else:
        def __init__(self, file: _WritableFileobj, protocol: int | None = ..., *, fix_imports: bool = ...) -> None: ...

    def dump(self, __obj: Any) -> None: ...
    def clear_memo(self) -> None: ...
    def persistent_id(self, obj: Any) -> Any: ...

class Unpickler:
    dispatch: ClassVar[dict[int, Callable[[Unpickler], None]]]  # undocumented, _Unpickler only

    if sys.version_info >= (3, 8):
        def __init__(
            self,
            file: _ReadableFileobj,
            *,
            fix_imports: bool = ...,
            encoding: str = ...,
            errors: str = ...,
            buffers: Iterable[Any] | None = ...,
        ) -> None: ...
    else:
        def __init__(
            self, file: _ReadableFileobj, *, fix_imports: bool = ..., encoding: str = ..., errors: str = ...
        ) -> None: ...

    def load(self) -> Any: ...
    def find_class(self, __module_name: str, __global_name: str) -> Any: ...
    def persistent_load(self, pid: Any) -> Any: ...

MARK: bytes
STOP: bytes
POP: bytes
POP_MARK: bytes
DUP: bytes
FLOAT: bytes
INT: bytes
BININT: bytes
BININT1: bytes
LONG: bytes
BININT2: bytes
NONE: bytes
PERSID: bytes
BINPERSID: bytes
REDUCE: bytes
STRING: bytes
BINSTRING: bytes
SHORT_BINSTRING: bytes
UNICODE: bytes
BINUNICODE: bytes
APPEND: bytes
BUILD: bytes
GLOBAL: bytes
DICT: bytes
EMPTY_DICT: bytes
APPENDS: bytes
GET: bytes
BINGET: bytes
INST: bytes
LONG_BINGET: bytes
LIST: bytes
EMPTY_LIST: bytes
OBJ: bytes
PUT: bytes
BINPUT: bytes
LONG_BINPUT: bytes
SETITEM: bytes
TUPLE: bytes
EMPTY_TUPLE: bytes
SETITEMS: bytes
BINFLOAT: bytes

TRUE: bytes
FALSE: bytes

# protocol 2
PROTO: bytes
NEWOBJ: bytes
EXT1: bytes
EXT2: bytes
EXT4: bytes
TUPLE1: bytes
TUPLE2: bytes
TUPLE3: bytes
NEWTRUE: bytes
NEWFALSE: bytes
LONG1: bytes
LONG4: bytes

# protocol 3
BINBYTES: bytes
SHORT_BINBYTES: bytes

# protocol 4
SHORT_BINUNICODE: bytes
BINUNICODE8: bytes
BINBYTES8: bytes
EMPTY_SET: bytes
ADDITEMS: bytes
FROZENSET: bytes
NEWOBJ_EX: bytes
STACK_GLOBAL: bytes
MEMOIZE: bytes
FRAME: bytes

if sys.version_info >= (3, 8):
    # Protocol 5
    BYTEARRAY8: bytes
    NEXT_BUFFER: bytes
    READONLY_BUFFER: bytes

def encode_long(x: int) -> bytes: ...  # undocumented
def decode_long(data: bytes) -> int: ...  # undocumented

# pure-Python implementations
_Pickler = Pickler  # undocumented
_Unpickler = Unpickler  # undocumented



class AESCCM:
    def __init__(self, key: bytes, tag_length: int | None) -> None: ...
    def decrypt(self, nonce: bytes, data: bytes, associated_data: bytes | None) -> bytes: ...
    def encrypt(self, nonce: bytes, data: bytes, associated_data: bytes | None) -> bytes: ...
    @classmethod
    def generate_key(cls, bit_length: int) -> bytes: ...

class AESGCM:
    def __init__(self, key: bytes) -> None: ...
    def decrypt(self, nonce: bytes, data: bytes, associated_data: bytes | None) -> bytes: ...
    def encrypt(self, nonce: bytes, data: bytes, associated_data: bytes | None) -> bytes: ...
    @classmethod
    def generate_key(cls, bit_length: int) -> bytes: ...

class ChaCha20Poly1305:
    def __init__(self, key: bytes) -> None: ...
    def decrypt(self, nonce: bytes, data: bytes, associated_data: bytes | None) -> bytes: ...
    def encrypt(self, nonce: bytes, data: bytes, associated_data: bytes | None) -> bytes: ...
    @classmethod
    def generate_key(cls) -> bytes: ...



from typing import Any, ContextManager, TextIO

from .ansitowin32 import StreamWrapper

orig_stdout: TextIO | None
orig_stderr: TextIO | None
wrapped_stdout: TextIO | StreamWrapper
wrapped_stderr: TextIO | StreamWrapper
atexit_done: bool

def reset_all() -> None: ...
def init(autoreset: bool = ..., convert: bool | None = ..., strip: bool | None = ..., wrap: bool = ...) -> None: ...
def deinit() -> None: ...
def colorama_text(*args: Any, **kwargs: Any) -> ContextManager[None]: ...
def reinit() -> None: ...
def wrap_stream(
    stream: TextIO, convert: bool | None, strip: bool | None, autoreset: bool, wrap: bool
) -> TextIO | StreamWrapper: ...



from typing import Any

from setuptools.command.setopt import option_base

def shquote(arg): ...

class alias(option_base):
    description: str
    command_consumes_arguments: bool
    user_options: Any
    boolean_options: Any
    args: Any
    remove: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...

def format_alias(name, aliases): ...



from _typeshed import Self
from abc import abstractmethod
from typing import Pattern, Text, TypeVar

_T = TypeVar("_T", bound=Version)

class Version:
    @abstractmethod
    def __init__(self, vstring: Text | None = ...) -> None: ...
    @abstractmethod
    def parse(self: Self, vstring: Text) -> Self: ...
    @abstractmethod
    def __str__(self) -> str: ...
    @abstractmethod
    def __cmp__(self: _T, other: _T | str) -> bool: ...

class StrictVersion(Version):
    version_re: Pattern[str]
    version: tuple[int, int, int]
    prerelease: tuple[Text, int] | None
    def __init__(self, vstring: Text | None = ...) -> None: ...
    def parse(self: Self, vstring: Text) -> Self: ...
    def __str__(self) -> str: ...  # noqa: Y029
    def __cmp__(self: _T, other: _T | str) -> bool: ...

class LooseVersion(Version):
    component_re: Pattern[str]
    vstring: Text
    version: tuple[Text | int, ...]
    def __init__(self, vstring: Text | None = ...) -> None: ...
    def parse(self: Self, vstring: Text) -> Self: ...
    def __str__(self) -> str: ...  # noqa: Y029
    def __cmp__(self: _T, other: _T | str) -> bool: ...



from typing import Any, Callable, Iterator, Pattern

ESCAPE: Pattern[str]
ESCAPE_ASCII: Pattern[str]
HAS_UTF8: Pattern[bytes]
ESCAPE_DCT: dict[str, str]
INFINITY: float

def py_encode_basestring(s: str) -> str: ...  # undocumented
def py_encode_basestring_ascii(s: str) -> str: ...  # undocumented

class JSONEncoder:
    item_separator: str
    key_separator: str

    skipkeys: bool
    ensure_ascii: bool
    check_circular: bool
    allow_nan: bool
    sort_keys: bool
    indent: int
    def __init__(
        self,
        *,
        skipkeys: bool = ...,
        ensure_ascii: bool = ...,
        check_circular: bool = ...,
        allow_nan: bool = ...,
        sort_keys: bool = ...,
        indent: int | None = ...,
        separators: tuple[str, str] | None = ...,
        default: Callable[..., Any] | None = ...,
    ) -> None: ...
    def default(self, o: Any) -> Any: ...
    def encode(self, o: Any) -> str: ...
    def iterencode(self, o: Any, _one_shot: bool = ...) -> Iterator[str]: ...



"""Analyze an entire project using pytype."""

import logging
import sys

import importlab.environment
import importlab.fs
import importlab.graph
import importlab.output

from pytype import io
from pytype.platform_utils import path_utils
from pytype.platform_utils import tempfile as compatible_tempfile
from pytype.tools import environment
from pytype.tools import tool_utils
from pytype.tools.analyze_project import config
from pytype.tools.analyze_project import environment as analyze_project_env
from pytype.tools.analyze_project import parse_args
from pytype.tools.analyze_project import pytype_runner


def main():
  parser = parse_args.make_parser()
  args = parser.parse_args(sys.argv[1:])

  if args.version:
    print(io.get_pytype_version())
    sys.exit(0)

  tool_utils.setup_logging_or_die(args.verbosity)

  if args.generate_config:
    config.generate_sample_config_or_die(args.generate_config,
                                         parser.pytype_single_args)
    sys.exit(0)

  conf = parser.config_from_defaults()
  # File options overwrite defaults.
  file_config = config.read_config_file_or_die(args.config)
  parser.postprocess(file_config, from_strings=True)
  conf.populate_from(file_config)
  # Command line arguments overwrite file options.
  conf.populate_from(args)
  conf.inputs -= conf.exclude
  if args.no_cache:
    conf.output = compatible_tempfile.mkdtemp()
  if not conf.pythonpath:
    conf.pythonpath = environment.compute_pythonpath(conf.inputs)
  logging.info('\n  '.join(['Configuration:'] + str(conf).split('\n')))

  if not conf.inputs:
    parser.parser.error('Need an input.')

  # Importlab needs the python exe, so we check it as early as possible.
  environment.check_python_exe_or_die(conf.python_version)

  typeshed = environment.initialize_typeshed_or_die()
  env = analyze_project_env.create_importlab_environment(conf, typeshed)
  print('Computing dependencies')
  import_graph = importlab.graph.ImportGraph.create(env, conf.inputs, trim=True)

  if args.tree:
    print('Source tree:')
    importlab.output.print_tree(import_graph)
    sys.exit(0)

  if args.unresolved:
    print('Unresolved dependencies:')
    for imp in sorted(import_graph.get_all_unresolved()):
      print(' ', imp.name)
    sys.exit(0)

  # Main usage mode: analyze the project file by file in dependency order.

  logging.info('Source tree:\n%s',
               importlab.output.formatted_deps_list(import_graph))
  tool_utils.makedirs_or_die(conf.output, 'Could not create output directory')
  with open(path_utils.join(conf.output, '.gitignore'), 'w') as f:
    f.write('# Automatically created by pytype\n*')
  deps = pytype_runner.deps_from_import_graph(import_graph)
  runner = pytype_runner.PytypeRunner(conf, deps)
  return runner.run()


if __name__ == '__main__':
  sys.exit(main())



import sys
from _typeshed import SupportsRichComparisonT
from typing import Callable, MutableSequence, Sequence, TypeVar, overload

_T = TypeVar("_T")

if sys.version_info >= (3, 10):
    @overload
    def bisect_left(
        a: Sequence[SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = ..., hi: int | None = ..., *, key: None = ...
    ) -> int: ...
    @overload
    def bisect_left(
        a: Sequence[_T],
        x: SupportsRichComparisonT,
        lo: int = ...,
        hi: int | None = ...,
        *,
        key: Callable[[_T], SupportsRichComparisonT] = ...,
    ) -> int: ...
    @overload
    def bisect_right(
        a: Sequence[SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = ..., hi: int | None = ..., *, key: None = ...
    ) -> int: ...
    @overload
    def bisect_right(
        a: Sequence[_T],
        x: SupportsRichComparisonT,
        lo: int = ...,
        hi: int | None = ...,
        *,
        key: Callable[[_T], SupportsRichComparisonT] = ...,
    ) -> int: ...
    @overload
    def insort_left(
        a: MutableSequence[SupportsRichComparisonT],
        x: SupportsRichComparisonT,
        lo: int = ...,
        hi: int | None = ...,
        *,
        key: None = ...,
    ) -> None: ...
    @overload
    def insort_left(
        a: MutableSequence[_T], x: _T, lo: int = ..., hi: int | None = ..., *, key: Callable[[_T], SupportsRichComparisonT] = ...
    ) -> None: ...
    @overload
    def insort_right(
        a: MutableSequence[SupportsRichComparisonT],
        x: SupportsRichComparisonT,
        lo: int = ...,
        hi: int | None = ...,
        *,
        key: None = ...,
    ) -> None: ...
    @overload
    def insort_right(
        a: MutableSequence[_T], x: _T, lo: int = ..., hi: int | None = ..., *, key: Callable[[_T], SupportsRichComparisonT] = ...
    ) -> None: ...

else:
    def bisect_left(
        a: Sequence[SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = ..., hi: int | None = ...
    ) -> int: ...
    def bisect_right(
        a: Sequence[SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = ..., hi: int | None = ...
    ) -> int: ...
    def insort_left(
        a: MutableSequence[SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = ..., hi: int | None = ...
    ) -> None: ...
    def insort_right(
        a: MutableSequence[SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = ..., hi: int | None = ...
    ) -> None: ...



from _typeshed import SupportsKeysAndGetItem
from collections.abc import Callable, Generator, Iterable
from typing import Any, ClassVar, Mapping

from ._utils import URIDict

_Schema = Mapping[str, Any]

# This class does not exist at runtime. Compatible classes are created at
# runtime by create().
class _Validator:
    VALIDATORS: ClassVar[dict[Any, Any]]
    META_SCHEMA: ClassVar[dict[Any, Any]]
    TYPE_CHECKER: Any
    @staticmethod
    def ID_OF(schema: _Schema) -> str: ...
    schema: _Schema
    resolver: Any
    format_checker: Any
    evolve: Any
    def __init__(self, schema: _Schema, resolver: Any | None = ..., format_checker: Any | None = ...) -> None: ...
    @classmethod
    def check_schema(cls, schema: _Schema) -> None: ...
    def iter_errors(self, instance, _schema: _Schema | None = ...) -> Generator[Any, None, None]: ...
    def descend(
        self, instance, schema: _Schema, path: Any | None = ..., schema_path: Any | None = ...
    ) -> Generator[Any, None, None]: ...
    def validate(self, *args, **kwargs) -> None: ...
    def is_type(self, instance, type): ...
    def is_valid(self, instance, _schema: _Schema | None = ...) -> bool: ...

def validates(version: str) -> Callable[..., Any]: ...
def create(
    meta_schema, validators=..., version: Any | None = ..., type_checker=..., id_of=..., applicable_validators=...
) -> type[_Validator]: ...
def extend(validator, validators=..., version: Any | None = ..., type_checker: Any | None = ...): ...

# At runtime these are fields that are assigned the return values of create() calls.
class Draft3Validator(_Validator): ...
class Draft4Validator(_Validator): ...
class Draft6Validator(_Validator): ...
class Draft7Validator(_Validator): ...
class Draft201909Validator(_Validator): ...
class Draft202012Validator(_Validator): ...

_Handler = Callable[[str], Any]

class RefResolver:
    referrer: dict[str, Any]
    cache_remote: Any
    handlers: dict[str, _Handler]
    store: URIDict
    def __init__(
        self,
        base_uri: str,
        referrer: dict[str, Any],
        store: SupportsKeysAndGetItem[str, str] | Iterable[tuple[str, str]] = ...,
        cache_remote: bool = ...,
        handlers: SupportsKeysAndGetItem[str, _Handler] | Iterable[tuple[str, _Handler]] = ...,
        urljoin_cache: Any | None = ...,
        remote_cache: Any | None = ...,
    ) -> None: ...
    @classmethod
    def from_schema(cls, schema: _Schema, id_of=..., *args, **kwargs): ...
    def push_scope(self, scope) -> None: ...
    def pop_scope(self) -> None: ...
    @property
    def resolution_scope(self): ...
    @property
    def base_uri(self): ...
    def in_scope(self, scope) -> None: ...
    def resolving(self, ref) -> None: ...
    def resolve(self, ref): ...
    def resolve_from_url(self, url): ...
    def resolve_fragment(self, document, fragment): ...
    def resolve_remote(self, uri): ...

def validate(instance: object, schema: _Schema, cls: type[_Validator] | None = ..., *args: Any, **kwargs: Any) -> None: ...
def validator_for(schema: _Schema | bool, default=...): ...
